@@ -270,6 +270,7 @@ pub static API_SPEC: LazyLock<Api> = LazyLock::new(|| Api {
                     vec![
                         "crypt".into(),
                         "force".into(),
+                        "ioctl".into(),
                         "lock".into(),
                         "mem".into(),
                         "pid".into(),
@@ -296,6 +297,7 @@ pub static API_SPEC: LazyLock<Api> = LazyLock::new(|| Api {
                     vec![
                         "crypt".into(),
                         "force".into(),
+                        "ioctl".into(),
                         "lock".into(),
                         "mem".into(),
                         "pid".into(),
@@ -322,6 +324,7 @@ pub static API_SPEC: LazyLock<Api> = LazyLock::new(|| Api {
                     vec![
                         "crypt".into(),
                         "force".into(),
+                        "ioctl".into(),
                         "lock".into(),
                         "mem".into(),
                         "pid".into(),
@@ -630,6 +633,24 @@ pub static API_SPEC: LazyLock<Api> = LazyLock::new(|| Api {
             op: Some(Operator::Set),
             ..Default::default()
         },
+        Method {
+            name: "force_no_symlinks".into(),
+            desc: "Force RESOLVE_NO_SYMLINKS at open(2) boundary".into(),
+            path: Some("trace/force_no_symlinks".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
+        Method {
+            name: "force_no_magiclinks".into(),
+            desc: "Force RESOLVE_NO_MAGICLINKS at open(2) boundary".into(),
+            path: Some("trace/force_no_magiclinks".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
         Method {
             name: "force_no_xdev".into(),
             desc: "Force RESOLVE_NO_XDEV at open(2) boundary".into(),
