@@ -11,12 +11,13 @@ use std::os::fd::AsFd;
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
-    sys::stat::{mkdirat, umask, Mode},
+    sys::stat::{mkdirat, Mode},
 };
 
 use crate::{
-    fs::FsFlags,
+    cookie::safe_umask,
     kernel::{syscall_path_handler, to_mode},
+    lookup::FsFlags,
     proc::proc_umask,
     req::{PathArgs, SysArg, UNotifyEventRequest},
 };
@@ -86,7 +87,7 @@ fn syscall_mkdir_handler(
     // SAFETY: Honour process' umask.
     // Note, the umask is per-thread here.
     // Note, POSIX ACLs may override this.
-    umask(mask);
+    safe_umask(mask);
 
     mkdirat(
         path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?,
