@@ -15,17 +15,13 @@ use ipnet::IpNet;
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
-    sys::{
-        socket::{getsockname, AddressFamily, SockaddrLike, SockaddrStorage},
-        stat::umask,
-    },
-    unistd::fchdir,
+    sys::socket::{getsockname, AddressFamily, SockaddrLike, SockaddrStorage},
 };
 
 use crate::{
-    cookie::safe_bind,
-    fs::CanonicalPath,
+    cookie::{safe_bind, safe_fchdir, safe_umask},
     info,
+    lookup::CanonicalPath,
     path::XPathBuf,
     proc::proc_umask,
     req::UNotifyEventRequest,
@@ -51,11 +47,11 @@ pub(crate) fn handle_bind(
         // 2. We cannot resolve symlinks in root or we risk TOCTOU!
         #[expect(clippy::disallowed_methods)]
         let dirfd = root.as_ref().unwrap().dir.as_ref().unwrap();
-        fchdir(dirfd)?;
+        safe_fchdir(dirfd)?;
 
         // SAFETY: Honour process' umask.
         // Note, the umask is per-thread here.
-        umask(mask);
+        safe_umask(mask);
 
         // SAFETY: bind() does not work through dangling
         // symbolic links even with SO_REUSEADDR. When called
