@@ -111,10 +111,14 @@ pub mod api;
 /// Assembly instruction decoder
 #[cfg(feature = "asm")]
 pub mod asm;
+/// Cgroup v2 management for resource limits
+pub mod cgroup;
 /// System call argument cookies
 pub mod cookie;
 /// Utilities to mask sensitive information in proc files
 pub(crate) mod mask;
+/// Safe mount interface
+pub mod mount;
 /// System call handlers
 #[macro_use]
 pub(crate) mod kernel;
@@ -229,7 +233,6 @@ use nix::{
     errno::Errno,
     sched::CloneFlags,
     sys::{
-        personality::Persona,
         resource::{getrlimit, Resource},
         signal::{sigaction, signal, SaFlags, SigAction, SigHandler, SigSet, Signal},
         socket::{socket, AddressFamily, SockFlag, SockType},
@@ -244,7 +247,7 @@ use crate::{
     confine::{
         apparmor_enabled, check_cross_memory_attach, check_unix_diag, is_coredump, lock_enabled,
         ns_enabled, seccomp_arch_native_name, selinux_enabled, selinux_enforced, vdso_list_calls,
-        SydPersona,
+        SydArch, SydPersona, SCMP_ARCH,
     },
     err::err2no,
     hash::{aes_ctr_info, check_setsockopt_serial_support, hmac_sha256_info, key_ring_validate},
@@ -566,8 +569,8 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
 
     // SAFETY: In libc we trust.
     // Note: nix version truncates unknown bits which we don't want.
-    let pers = match Errno::result(unsafe { libc::personality(0xFFFFFFFF) }) {
-        Ok(n) => SydPersona(Persona::from_bits_retain(n)).to_string(),
+    let pers = match SydPersona::get() {
+        Ok(pers) => pers.to_string(),
         Err(errno) => format!("? (error: {errno})"),
     };
 
@@ -577,6 +580,41 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
         env!("SYD_TARGET_POINTER_WIDTH")
     )?;
 
+    // Log supported architectures.
+    let arch = SCMP_ARCH
+        .iter()
+        .map(SydArch::from)
+        .map(|arch| arch.to_string())
+        .collect::<Vec<_>>()
+        .join(", ");
+    printfln!("Architectures: {arch}")?;
+
+    // Log architectures with ipc(2), socketcall(2) multiplexer support.
+    let mut has_ipc = Vec::new();
+    let mut has_socketcall = Vec::new();
+    for arch in SCMP_ARCH.iter().map(SydArch::from) {
+        if arch.has_ipc() {
+            has_ipc.push(arch.to_string());
+        }
+        if arch.has_socketcall() {
+            has_socketcall.push(arch.to_string());
+        }
+    }
+    if !has_ipc.is_empty() {
+        let plurals = if has_ipc.len() > 1 { "s" } else { "" };
+        let verbone = if has_ipc.len() == 1 { "s" } else { "" };
+        let has_ipc = has_ipc.join(", ");
+        printfln!("Architecture{plurals} {has_ipc} support{verbone} ipc(2) multiplexer.")?;
+    }
+    if !has_socketcall.is_empty() {
+        let plurals = if has_socketcall.len() > 1 { "s" } else { "" };
+        let verbone = if has_socketcall.len() == 1 { "s" } else { "" };
+        let has_socketcall = has_socketcall.join(", ");
+        printfln!(
+            "Architecture{plurals} {has_socketcall} support{verbone} socketcall(2) multiplexer."
+        )?;
+    }
+
     printfln!(
         "CPU: {} ({} cores), {}-endian",
         num_cpus::get(),
