@@ -28,7 +28,7 @@ use std::{
     ffi::{CStr, OsStr, OsString},
     fmt,
     fs::File,
-    io::{BufReader, IoSlice, IoSliceMut, Read, Seek, SeekFrom},
+    io::{BufReader, Cursor, IoSlice, IoSliceMut, Read, Seek, SeekFrom},
     num::NonZeroUsize,
     ops::Range,
     os::{
@@ -36,7 +36,6 @@ use std::{
         unix::ffi::OsStrExt,
     },
     ptr::NonNull,
-    sync::LazyLock,
 };
 
 use bitflags::bitflags;
@@ -44,7 +43,7 @@ use btoi::{btoi, btoi_radix};
 use memchr::{arch::all::is_prefix, memrchr};
 use nix::{
     errno::Errno,
-    fcntl::{readlinkat, OFlag, ResolveFlag},
+    fcntl::OFlag,
     libc::pid_t,
     sys::{
         stat::Mode,
@@ -61,16 +60,19 @@ use procfs_core::{
     },
     CryptoTable, FromBufRead,
 };
+use serde::{ser::SerializeMap, Serialize, Serializer};
 
 use crate::{
-    compat::{getdents64, statx, STATX_INO},
+    compat::{getdents64, openat2, statx, ResolveFlag, STATX_INO},
     config::*,
     confine::SydMemoryMap,
     cookie::{PROCMAP_QUERY_COOKIE_ARG3, PROCMAP_QUERY_COOKIE_ARG4, PROCMAP_QUERY_COOKIE_ARG5},
     elf::ElfType,
     err::{err2no, proc_error_to_errno},
-    fs::{is_open_fd, parse_fd, read_all, safe_open, write_all, AT_BADFD},
+    fs::{is_empty_file, is_open_fd, parse_fd, readlinkat, AT_BADFD},
     hash::{SydHashMap, SydHashSet, SydIndexSet},
+    io::{read_all, read_buf, write_all},
+    lookup::{safe_open, safe_open_how_abs},
     parsers::{
         map_result,
         proc::{
@@ -82,6 +84,7 @@ use crate::{
     path::{XPathBuf, PATH_MAX},
     retry::retry_on_eintr,
     rng::randint,
+    sandbox::Sandbox,
     sigset::SydSigSet,
     XPath,
 };
@@ -241,7 +244,7 @@ pub struct ProcmapQuery {
 impl Default for ProcmapQuery {
     fn default() -> Self {
         Self {
-            size: std::mem::size_of::<Self>() as u64,
+            size: size_of::<Self>() as u64,
             query_flags: 0,
             query_addr: 0,
             vma_start: 0,
@@ -387,16 +390,33 @@ impl Default for Status {
     }
 }
 
+impl Serialize for Status {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut map = serializer.serialize_map(Some(7))?;
+        map.serialize_entry("umask", &self.umask.bits())?;
+        map.serialize_entry("pid", &self.pid.as_raw())?;
+        map.serialize_entry("sig_pending_thread", &self.sig_pending_thread)?;
+        map.serialize_entry("sig_pending_process", &self.sig_pending_process)?;
+        map.serialize_entry("sig_blocked", &self.sig_blocked)?;
+        map.serialize_entry("sig_ignored", &self.sig_ignored)?;
+        map.serialize_entry("sig_caught", &self.sig_caught)?;
+        map.end()
+    }
+}
+
 /// Safe `OFlags` to use when opening proc(5) files.
 pub const OFLAG_PROC: OFlag =
     OFlag::from_bits_retain(libc::O_RDONLY | libc::O_CLOEXEC | libc::O_NOFOLLOW | libc::O_NOCTTY);
 
 /// Safe `ResolveFlag` to use when opening proc(5) files.
 pub const RFLAG_PROC: ResolveFlag = ResolveFlag::from_bits_retain(
-    libc::RESOLVE_NO_MAGICLINKS
-        | libc::RESOLVE_NO_SYMLINKS
-        | libc::RESOLVE_BENEATH
-        | libc::RESOLVE_NO_XDEV,
+    ResolveFlag::RESOLVE_NO_MAGICLINKS.bits()
+        | ResolveFlag::RESOLVE_NO_SYMLINKS.bits()
+        | ResolveFlag::RESOLVE_BENEATH.bits()
+        | ResolveFlag::RESOLVE_NO_XDEV.bits(),
 );
 
 // Function to map proc(5) errno ENOENT to ESRCH to reduce confusion.
@@ -474,7 +494,8 @@ fn usize_to_ne_bytes(value: usize, sizeof_ptr: usize) -> Vec<u8> {
 
 /// Return the stat of the given process.
 pub fn proc_stat(pid: Pid) -> Result<Stat, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/stat".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"stat");
 
     // Safely open proc(5) file.
@@ -486,7 +507,8 @@ pub fn proc_stat(pid: Pid) -> Result<Stat, Errno> {
 
 /// Return the memory stat of the given process.
 pub fn proc_statm(pid: Pid) -> Result<Statm, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/statm".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"statm");
 
     // Safely open proc(5) file.
@@ -504,7 +526,8 @@ pub fn proc_status(pid: Pid) -> Result<Status, Errno> {
 
 /// Open a FD to /proc/$pid/status and return an OwnedFd.
 pub fn proc_status_open(pid: Pid) -> Result<OwnedFd, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/status".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"status");
 
     safe_open_proc(&path).map_err(proc_errno)
@@ -530,7 +553,7 @@ pub fn proc_tgid(tid: Pid) -> Result<Pid, Errno> {
 
     // Read up to 192 bytes.
     let mut data = [0u8; 192];
-    let nread = read_all(fd, &mut data)?;
+    let nread = read_buf(fd, &mut data)?;
 
     map_result(parse_status_tgid(&data[..nread]))
 }
@@ -543,7 +566,7 @@ pub fn proc_umask(pid: Pid) -> Result<Mode, Errno> {
 
     // Read up to 160 bytes.
     let mut data = [0u8; 160];
-    let nread = read_all(fd, &mut data)?;
+    let nread = read_buf(fd, &mut data)?;
 
     map_result(parse_status_umask(&data[..nread]))
 }
@@ -571,17 +594,13 @@ pub fn proc_stack_start(pid: Pid) -> Result<u64, Errno> {
 ///
 /// If the process is "running", it returns `Errno::EBUSY`.
 pub fn proc_stack_pointer(pid: Pid) -> Result<u64, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/syscall".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"syscall");
 
     // Safely open proc(5) file.
-    let mut file = safe_open_proc(&path)
-        .map(File::from)
-        .map(BufReader::new)
-        .map_err(proc_errno)?;
-
-    let mut data = Vec::with_capacity(128);
-    file.read_to_end(&mut data).or(Err(Errno::EIO))?;
+    let file = safe_open_proc(&path).map_err(proc_errno)?;
+    let data = read_all(file)?;
 
     // Check if the contents equal "running".
     if is_prefix(&data, b"running") {
@@ -614,23 +633,22 @@ pub fn proc_stack_pointer(pid: Pid) -> Result<u64, Errno> {
 
 /// Get the auxiliary vector of the given `Pid`.
 pub fn proc_auxv(pid: Pid) -> Result<SydHashMap<u64, u64>, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/auxv".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"auxv");
 
     // Safely open proc(5) file.
-    let mut file = safe_open_proc(&path).map(File::from).map_err(proc_errno)?;
+    let file = safe_open_proc(&path).map(File::from).map_err(proc_errno)?;
 
     let mut map = SydHashMap::default();
 
-    let mut buf = Vec::new();
-    let bytes_read = file.read_to_end(&mut buf).map_err(|err| err2no(&err))?;
-    if bytes_read == 0 {
-        // some kernel processes won't have any data for their auxv file
+    let buf = read_all(&file)?;
+    if buf.is_empty() {
+        // Some kernel processes won't have any data for their auxv file.
         return Ok(map);
     }
-    buf.truncate(bytes_read);
-    let mut file = std::io::Cursor::new(buf);
 
+    let mut file = Cursor::new(buf);
     let mut buf = 0usize.to_ne_bytes();
     loop {
         file.read_exact(&mut buf).map_err(|err| err2no(&err))?;
@@ -650,17 +668,15 @@ pub fn proc_auxv(pid: Pid) -> Result<SydHashMap<u64, u64>, Errno> {
 ///
 /// This function reads the symbolic link `/proc/<pid>/cwd` to determine the CWD.
 pub fn proc_cwd(pid: Pid) -> Result<XPathBuf, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
-    path.push(b"cwd");
-    readlinkat(PROC_FILE(), &path)
-        .map(XPathBuf::from)
-        .map_err(proc_errno)
+    let path = XPathBuf::from_cwd(pid)?;
+    readlinkat(PROC_FILE(), &path).map_err(proc_errno)
 }
 
 /// Retrieves the command name (comm) of the specified process ID (PID)
 /// as a single string.
 pub fn proc_comm(pid: Pid) -> Result<XPathBuf, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/comm".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"comm");
 
     // Safely open proc(5) file.
@@ -668,7 +684,7 @@ pub fn proc_comm(pid: Pid) -> Result<XPathBuf, Errno> {
 
     // Read up to 16 characters or until EOF.
     let mut comm = [0u8; 16];
-    let mut nread = read_all(file, &mut comm)?;
+    let mut nread = read_buf(file, &mut comm)?;
 
     // Remove the trailing NUL-byte and return an `XPathBuf`.
     let idx = nread.saturating_sub(1);
@@ -688,7 +704,8 @@ pub fn proc_comm(pid: Pid) -> Result<XPathBuf, Errno> {
 /// bytes (`'\0'`) with spaces to format the command line as a readable string.
 pub fn proc_cmdline(pid: Pid) -> Result<XPathBuf, Errno> {
     // Construct path to the appropriate cmdline file.
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/cmdline".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"cmdline");
 
     // Safely open proc(5) file.
@@ -697,7 +714,7 @@ pub fn proc_cmdline(pid: Pid) -> Result<XPathBuf, Errno> {
     // Read up to 256 bytes.
     const LIMIT: usize = 256;
     let mut data = [0u8; LIMIT];
-    let nread = read_all(file, &mut data)?;
+    let nread = read_buf(file, &mut data)?;
 
     let mut data = data.to_vec();
     // Determine if EOF was reached or if we hit the limit
@@ -738,7 +755,8 @@ pub fn proc_maps(pid: Pid) -> Result<Vec<SydMemoryMap>, Errno> {
 
 /// Open a FD to _proc_pid_maps_(5) and return an OwnedFd.
 pub fn proc_maps_open(pid: Pid) -> Result<OwnedFd, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/maps".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"maps");
 
     safe_open_proc(&path).map_err(proc_errno)
@@ -756,7 +774,8 @@ pub fn proc_maps_read(fd: OwnedFd) -> Result<Vec<SydMemoryMap>, Errno> {
 
 /// Return the memory maps of the given process.
 pub fn proc_smaps(pid: Pid) -> Result<Vec<SydMemoryMap>, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/smaps".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"smaps");
 
     // Safely open proc(5) file.
@@ -776,7 +795,9 @@ pub fn proc_smaps(pid: Pid) -> Result<Vec<SydMemoryMap>, Errno> {
 /// This is more efficient than using `proc_smaps` and iterating over maps.
 /// See: https://www.kernel.org/doc/Documentation/ABI/testing/procfs-smaps_rollup
 pub fn proc_smaps_rollup(pid: Pid) -> Result<SydMemoryMap, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/smaps_rollup".len())
+        .or(Err(Errno::ENOMEM))?;
     path.push(b"smaps_rollup");
 
     // Safely open proc(5) file.
@@ -841,7 +862,7 @@ pub fn proc_pipemax_open() -> Result<OwnedFd, Errno> {
 pub fn proc_pipemax_read<Fd: AsFd>(fd: Fd) -> Result<libc::c_int, Errno> {
     // Read up to 24 bytes.
     let mut data = [0u8; 24];
-    let nread = read_all(fd, &mut data)?;
+    let nread = read_buf(fd, &mut data)?;
 
     btoi::<libc::c_int>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
 }
@@ -855,7 +876,7 @@ pub fn proc_fs_file_max() -> Result<u64, Errno> {
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
-    let nread = read_all(fd, &mut data)?;
+    let nread = read_buf(fd, &mut data)?;
 
     btoi::<u64>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
 }
@@ -869,7 +890,7 @@ pub fn proc_fs_nr_open() -> Result<u64, Errno> {
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
-    let nread = read_all(fd, &mut data)?;
+    let nread = read_buf(fd, &mut data)?;
 
     btoi::<u64>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
 }
@@ -888,7 +909,7 @@ pub fn proc_mmap_min_addr() -> Result<u64, Errno> {
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
-    let nread = read_all(fd, &mut data)?;
+    let nread = read_buf(fd, &mut data)?;
 
     btoi::<u64>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
 }
@@ -923,7 +944,7 @@ pub fn proc_kernel_randomize_va_space() -> Result<u8, Errno> {
 
     // Read up to 1 byte + 1 byte for \n.
     let mut data = [0u8; 2];
-    let nread = read_all(fd, &mut data)?;
+    let nread = read_buf(fd, &mut data)?;
 
     btoi::<u8>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
 }
@@ -1330,7 +1351,8 @@ pub fn proc_environ(pid: Pid) -> Result<SydHashMap<OsString, OsString>, Errno> {
 
 /// Open a FD to _proc_pid_environ_(5) and return an OwnedFd.
 pub fn proc_environ_open(pid: Pid) -> Result<OwnedFd, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/environ".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"environ");
 
     safe_open_proc(&path).map_err(proc_errno)
@@ -1338,11 +1360,7 @@ pub fn proc_environ_open(pid: Pid) -> Result<OwnedFd, Errno> {
 
 /// Parses /proc/pid/environ from given Fd and returns a HashMap.
 pub fn proc_environ_read(fd: OwnedFd) -> Result<SydHashMap<OsString, OsString>, Errno> {
-    let mut buf = Vec::new();
-    File::from(fd)
-        .read_to_end(&mut buf)
-        .map_err(|err| err2no(&err))?;
-
+    let buf = read_all(fd)?;
     let mut map = SydHashMap::default();
     for slice in buf.split(|b| *b == 0) {
         // slice will be in the form key=var, so split on the first equals sign.
@@ -1353,7 +1371,6 @@ pub fn proc_environ_read(fd: OwnedFd) -> Result<SydHashMap<OsString, OsString>,
                 OsStr::from_bytes(v).to_os_string(),
             );
         };
-        //let env = OsStr::from_bytes(slice);
     }
 
     Ok(map)
@@ -1450,7 +1467,9 @@ pub fn proc_crypto_read(fd: OwnedFd) -> Result<CryptoTable, Errno> {
 ///
 /// Returns the inodes of all UNIX domain sockets.
 pub fn proc_unix_inodes(pid: Pid) -> Result<SydHashSet<u64>, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/net/unix".len())
+        .or(Err(Errno::ENOMEM))?;
     path.push(b"net");
     path.push(b"unix");
 
@@ -1491,32 +1510,9 @@ pub fn proc_unix_inodes(pid: Pid) -> Result<SydHashSet<u64>, Errno> {
 pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
     // This function requires CONFIG_CROSS_MEMORY_ATTACH,
     // unless Syd is explicitly configured to fallback.
-    static FORCE_PROC: LazyLock<bool> =
-        LazyLock::new(|| std::env::var_os(ENV_NO_CROSS_MEMORY_ATTACH).is_some());
-    static PROC_MEM_F: LazyLock<bool> =
-        LazyLock::new(|| std::env::var_os(ENV_PROC_PID_MEM_FALLBACK).is_some());
-    proc_set_at_secure2(
-        pid,
-        elf_type,
-        *FORCE_PROC,
-        *PROC_MEM_F,
-        *HAVE_CROSS_MEMORY_ATTACH,
-    )
-}
-
-/// Sets the AT_SECURE value to 1 in the auxiliary vector of the
-/// specified process.
-pub fn proc_set_at_secure2(
-    pid: Pid,
-    elf_type: ElfType,
-    force_proc: bool,
-    proc_mem_f: bool,
-    have_cma: bool,
-) -> Result<(), Errno> {
-    // This function requires CONFIG_CROSS_MEMORY_ATTACH,
-    // unless Syd is explicitly configured to fallback.
-    let mut proc_mem = if force_proc || (proc_mem_f && !have_cma) {
-        let mut path = XPathBuf::from_pid(pid);
+    let mut proc_mem = if Sandbox::use_proc_pid_mem() {
+        let mut path = XPathBuf::from_pid(pid)?;
+        path.try_reserve(b"/mem".len()).or(Err(Errno::ENOMEM))?;
         path.push(b"mem");
 
         let mut flags = OFLAG_PROC;
@@ -1541,7 +1537,7 @@ pub fn proc_set_at_secure2(
             Some(file) => {
                 file.seek(SeekFrom::Start(base_addr as u64))
                     .map_err(|err| err2no(&err))?;
-                read_all(file, buf)
+                read_buf(file, buf)
             }
             None => {
                 let local_len = buf.len();
@@ -1791,7 +1787,8 @@ pub fn proc_set_at_secure2(
 pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
     let pid = pid.unwrap_or_else(getpid);
 
-    let mut dir = XPathBuf::from_pid(pid);
+    let mut dir = XPathBuf::from_pid(pid)?;
+    dir.try_reserve(b"/fd".len()).or(Err(Errno::ENOMEM))?;
     dir.push(b"fd");
 
     let dir = safe_open(
@@ -1801,8 +1798,9 @@ pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
         RFLAG_PROC,
     )?;
 
-    let mut dot = 0u8;
     let mut res = vec![];
+    let mut seen_dot = false;
+    let mut seen_dotdot = false;
     loop {
         let mut entries = match getdents64(&dir, DIRENT_BUF_SIZE) {
             Ok(entries) => entries,
@@ -1811,13 +1809,17 @@ pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
         };
 
         for entry in &mut entries {
-            #[expect(clippy::arithmetic_side_effects)]
-            if dot < 2 && entry.is_dot() {
-                dot += 1;
+            if !seen_dot && entry.is_dot() {
+                seen_dot = true;
                 continue;
             }
-            let fd = parse_fd(XPath::from_bytes(entry.name_bytes()))?;
-            let target = readlinkat(&dir, entry.name_bytes()).map(XPathBuf::from)?;
+            if !seen_dotdot && entry.is_dotdot() {
+                seen_dotdot = true;
+                continue;
+            }
+            let entry = XPath::from_bytes(entry.name_bytes());
+            let fd = parse_fd(entry)?;
+            let target = readlinkat(&dir, entry)?;
 
             res.push((fd, target));
         }
@@ -1828,7 +1830,8 @@ pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
 
 /// Return the max open files soft limit for the given process.
 pub fn proc_max_open_files(pid: Pid) -> Result<LimitValue, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/limits".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"limits");
 
     // Safely open proc(5) file.
@@ -2005,13 +2008,33 @@ pub fn proc_kernel_taint() -> Result<KernelTaintFlags, Errno> {
 
     // Read up to 24 bytes + 1 byte for \n.
     let mut data = [0u8; 25];
-    let nread = read_all(fd, &mut data)?;
+    let nread = read_buf(fd, &mut data)?;
 
     // Parse decimal -> u64, then map to flags (retain unknown higher bits).
     let val = btoi::<u64>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))?;
     Ok(KernelTaintFlags::from_bits_retain(val))
 }
 
+/// Write the given $data to /proc/$pid/$name.
+pub fn proc_write(pid: Pid, name: &[u8], data: &[u8]) -> Result<(), Errno> {
+    let mut pfd = XPathBuf::from("/proc");
+    pfd.push_pid(pid);
+    pfd.push(name);
+
+    let how = safe_open_how_abs(OFlag::O_WRONLY, ResolveFlag::empty());
+
+    // This function runs early at startup before syd::config::proc_init.
+    #[expect(clippy::disallowed_methods)]
+    let fd = retry_on_eintr(|| openat2(AT_BADFD, &pfd, how))?;
+
+    // Check the file is an empty file or bail out.
+    if !is_empty_file(&fd).unwrap_or(false) {
+        return Err(Errno::EBADFD);
+    }
+
+    write_all(&fd, data)
+}
+
 #[cfg(test)]
 mod tests {
     use std::os::unix::fs::PermissionsExt;
@@ -2083,7 +2106,7 @@ mod tests {
     fn test_procmap_query_default_struct() {
         // Default values should be correct and size set properly.
         let q = ProcmapQuery::default();
-        assert_eq!(q.size as usize, std::mem::size_of::<ProcmapQuery>());
+        assert_eq!(q.size as usize, size_of::<ProcmapQuery>());
         assert_eq!(q.query_flags, 0);
         assert_eq!(q.query_addr, 0);
         assert_eq!(q.vma_start, 0);
@@ -2355,7 +2378,7 @@ mod tests {
             None,
         )
         .unwrap();
-        assert_eq!(q.size as usize, std::mem::size_of::<ProcmapQuery>());
+        assert_eq!(q.size as usize, size_of::<ProcmapQuery>());
     }
 
     #[test]
@@ -2433,15 +2456,18 @@ mod tests {
         }
 
         // Returned paths should exist on filesystem and have exec bit.
+        // ld.so may not have the exec bit on some systems, e.g. i386.
         let bins = proc_executables(Pid::this()).unwrap();
-        for bin in bins {
+        for (idx, bin) in bins.into_iter().enumerate() {
             let path = bin.path;
             let md = std::fs::metadata(&path).expect("path does not exist");
-            let perms = md.permissions().mode();
-            assert!(
-                perms & 0o111 != 0,
-                "file {path} is not executable (mode {perms:o})",
-            );
+            if idx == 0 {
+                let perms = md.permissions().mode();
+                assert!(
+                    perms & 0o111 != 0,
+                    "file {path} is not executable (mode {perms:o})",
+                );
+            }
         }
     }
 
