@@ -19,7 +19,6 @@
 use std::{
     ffi::CStr,
     fs::{File, Permissions},
-    io::{self, Read},
     ops::{Deref, DerefMut},
     os::unix::{fs::PermissionsExt as _, io::AsRawFd, process::CommandExt},
     process::Command,
@@ -32,10 +31,9 @@ use nix::{
         c_int, c_uint, F_SEAL_FUTURE_WRITE, F_SEAL_GROW, F_SEAL_SEAL, F_SEAL_SHRINK, F_SEAL_WRITE,
         MFD_ALLOW_SEALING, MFD_CLOEXEC, MFD_EXEC, MFD_NOEXEC_SEAL,
     },
-    sys::memfd::MFdFlags,
 };
 
-use crate::err::err2no;
+use crate::{compat::MFdFlags, config::ENV_SKIP_SCMP, err::err2no, io::ReadFd, path::XPathBuf};
 
 // SAFETY: The provided slice is nul-terminated and does not contain any interior nul bytes. On Rust
 // 1.64 and later (rust-lang/rust#99977), these required invariants are checked at compile time.
@@ -96,7 +94,9 @@ pub fn ensure_sealed() -> Result<(), Errno> {
             Some(val) => command.env("SYD_RUST_BACKTRACE", val),
             None => command.env_remove("SYD_RUST_BACKTRACE"),
         };
-        command.env("RUST_BACKTRACE", "0");
+        if std::env::var_os(ENV_SKIP_SCMP).is_none() {
+            command.env("RUST_BACKTRACE", "0");
+        }
 
         Err(err2no(&command.exec()))
     }
@@ -124,12 +124,17 @@ impl SealedCommand {
     ///
     /// An error is returned if `memfd_create(2)` fails, the `fcntl(2)` `F_GET_SEALS` or
     /// `F_ADD_SEALS` commands fail, or copying from `program` to the anonymous file fails.
-    pub fn new<R: Read>(program: &mut R) -> Result<Self, Errno> {
+    pub fn new<Fd>(program: &mut Fd) -> Result<Self, Errno>
+    where
+        Fd: ReadFd,
+    {
         let mut memfd = OPTIONS.create()?;
-        io::copy(program, &mut memfd).or(Err(Errno::EIO))?;
+        crate::io::copy(program, &mut memfd)?;
         OPTIONS.seal(&mut memfd)?;
 
-        let mut comm = Command::new(format!("/proc/self/fd/{}", memfd.as_raw_fd()));
+        let mut comm = XPathBuf::from("/proc/thread-self/fd");
+        comm.push_fd(memfd.as_raw_fd());
+        let mut comm = Command::new(comm);
         comm.arg0("syd");
 
         Ok(Self {
@@ -366,23 +371,16 @@ impl SealOptions {
 
     /// Create an anonymous file, copy the contents of `reader` to it, and seal it.
     ///
-    /// This is equivalent to:
-    /// ```
-    /// # let options = syd::seal::SealOptions::new();
-    /// # let reader: &mut &[u8] = &mut &[][..];
-    /// let mut file = options.create()?;
-    /// std::io::copy(reader, &mut file)?;
-    /// options.seal(&mut file)?;
-    /// # Ok::<(), std::io::Error>(())
-    /// ```
-    ///
     /// # Errors
     ///
-    /// This method returns an error when any of [`SealOptions::create`], [`std::io::copy`], or
+    /// This method returns an error when any of [`SealOptions::create`], [`syd::io::copy`], or
     /// [`SealOptions::seal`] fail.
-    pub fn copy_and_seal<R: Read>(&self, reader: &mut R) -> Result<File, Errno> {
+    pub fn copy_and_seal<Fd>(&self, reader: &mut Fd) -> Result<File, Errno>
+    where
+        Fd: ReadFd,
+    {
         let mut file = self.create()?;
-        io::copy(reader, &mut file).or(Err(Errno::EIO))?;
+        crate::io::copy(reader, &mut file)?;
         self.seal(&mut file)?;
         Ok(file)
     }
@@ -482,7 +480,7 @@ impl SealOptions {
 }
 
 fn memfd_create(name: &CStr, flags: c_uint) -> Result<File, Errno> {
-    nix::sys::memfd::memfd_create(name, MFdFlags::from_bits_retain(flags)).map(File::from)
+    nix::sys::memfd::memfd_create(name, MFdFlags::from_bits_retain(flags).into()).map(File::from)
 }
 
 fn fcntl_get_seals(file: &File) -> Result<c_int, Errno> {
@@ -495,12 +493,9 @@ fn fcntl_add_seals(file: &File, arg: c_int) -> Result<(), Errno> {
 
 #[cfg(test)]
 mod test {
-    use std::os::unix::fs::PermissionsExt as _;
+    use std::{fs::File, os::unix::fs::PermissionsExt as _};
 
-    use super::{
-        c_int, SealOptions, F_SEAL_EXEC, F_SEAL_FUTURE_WRITE, F_SEAL_GROW, F_SEAL_SEAL,
-        F_SEAL_SHRINK, F_SEAL_WRITE, MFD_ALLOW_SEALING, MFD_CLOEXEC, MFD_EXEC, MFD_NOEXEC_SEAL,
-    };
+    use super::*;
 
     #[test]
     fn new() {
@@ -643,15 +638,16 @@ mod test {
 
     #[test]
     fn executable() {
+        let mut null = File::open("/dev/null").unwrap();
         let file = SealOptions::new()
             .executable(false)
-            .copy_and_seal(&mut &[][..])
+            .copy_and_seal(&mut null)
             .unwrap();
         assert_eq!(file.metadata().unwrap().permissions().mode() & 0o111, 0);
 
         let file = SealOptions::new()
             .executable(true)
-            .copy_and_seal(&mut &[][..])
+            .copy_and_seal(&mut null)
             .unwrap();
         assert_eq!(file.metadata().unwrap().permissions().mode() & 0o111, 0o111);
     }
