@@ -1,9 +1,7 @@
 use std::{
-    cmp::Ordering,
     ffi::CString,
-    hint::unreachable_unchecked,
     os::{
-        fd::{AsFd, AsRawFd, BorrowedFd, FromRawFd, OwnedFd},
+        fd::{AsFd, AsRawFd, BorrowedFd, FromRawFd, IntoRawFd, OwnedFd},
         unix::io::RawFd,
     },
 };
@@ -21,8 +19,7 @@ use nix::{
 };
 
 use crate::{
-    compat::{waitid, WaitStatus},
-    config::CHLD_STACK_SIZE,
+    compat::{clone3, waitid, CloneArgs, WaitStatus},
     fs::pidfd_getfd,
     retry::retry_on_eintr,
     rng::duprand,
@@ -50,9 +47,6 @@ impl Command {
         let exe_file = self.exe_file.take().ok_or(Errno::EFAULT)?;
         let exe_args = self.exe_args.take().ok_or(Errno::EFAULT)?;
 
-        let mut pid_fd: libc::c_int = -1;
-        let clone_flags = libc::SIGCHLD | libc::CLONE_PIDFD;
-
         // SAFETY: Child owns its copy of the pipes,
         // and is responsible for closing them.
         let seccomp_pipefd = unsafe {
@@ -80,23 +74,35 @@ impl Command {
         });
         let child_info_ptr: *mut libc::c_void = Box::into_raw(child_info) as *mut libc::c_void;
 
-        // 2M stack by default, see config.rs.
-        let mut stack = [0u8; CHLD_STACK_SIZE];
-
-        // SAFETY: nix's clone does not support CLONE_PIDFD,
-        // so we use libc::clone instead.
-        let child = unsafe {
-            let ptr = stack.as_mut_ptr().add(stack.len());
-            let ptr_aligned = ptr.sub(ptr as usize % 16);
-            libc::clone(
-                child::child_after_clone as extern "C" fn(*mut libc::c_void) -> libc::c_int,
-                ptr_aligned as *mut libc::c_void,
-                clone_flags,
-                child_info_ptr,
-                &mut pid_fd,
-            )
+        let mut pid_fd: libc::c_int = -1;
+        let mut clone_args = CloneArgs {
+            flags: libc::CLONE_PIDFD as u64,
+            pidfd: (&raw mut pid_fd) as u64,
+            exit_signal: libc::SIGCHLD as u64,
+            ..Default::default()
         };
 
+        let child = match clone3(&mut clone_args) {
+            Ok(child) => child,
+            Err(errno) => {
+                // SAFETY: Reconstruct and drop.
+                drop(unsafe { Box::from_raw(child_info_ptr as *mut ChildInfo) });
+                return Err(errno);
+            }
+        };
+
+        if child.as_raw() == 0 {
+            child::child_after_clone(child_info_ptr);
+        }
+
+        // SAFETY: Reconstruct and forget pipe fds,
+        // after_start is going to handle them.
+        let child_info = unsafe { Box::from_raw(child_info_ptr as *mut ChildInfo) };
+        let _ = child_info.seccomp_pipefd.0 .0.into_raw_fd();
+        let _ = child_info.seccomp_pipefd.0 .1.into_raw_fd();
+        let _ = child_info.seccomp_pipefd.1 .0.into_raw_fd();
+        let _ = child_info.seccomp_pipefd.1 .1.into_raw_fd();
+
         // SAFETY: Randomize the pid FD for hardening.
         // The created fd is O_CLOEXEC too, and we'll
         // send the number to the child to close it.
@@ -106,37 +112,23 @@ impl Command {
         // SAFETY: duprand returns a valid FD on success.
         let pid_fd = unsafe { BorrowedFd::borrow_raw(pid_fd) };
 
-        match child.cmp(&0) {
-            Ordering::Less => {
-                // SAFETY: Reconstruct and drop.
-                let _ = unsafe { Box::from_raw(child_info_ptr as *mut ChildInfo) };
-                Err(Errno::last())
-            }
-            Ordering::Greater => {
-                let child = Pid::from_raw(child);
+        let seccomp_fd = match self.after_start(child, pid_fd) {
+            Ok(seccomp_fd) => seccomp_fd,
+            Err(e) => loop {
+                match waitid(Id::PIDFd(pid_fd.as_fd()), WaitPidFlag::WEXITED) {
+                    Ok(WaitStatus::Exited(_, errno)) => return Err(Errno::from_raw(errno)),
+                    Err(Errno::EINTR) => {}
+                    _ => return Err(e),
+                }
+            },
+        };
 
-                let seccomp_fd = match self.after_start(child, pid_fd) {
-                    Ok(seccomp_fd) => seccomp_fd,
-                    Err(e) => loop {
-                        match waitid(Id::PIDFd(pid_fd.as_fd()), WaitPidFlag::WEXITED) {
-                            Ok(WaitStatus::Exited(_, errno)) => return Err(Errno::from_raw(errno)),
-                            Err(Errno::EINTR) => {}
-                            _ => return Err(e),
-                        }
-                    },
-                };
-
-                Ok(Child {
-                    pid: child.into(),
-                    pid_fd: pid_fd.as_raw_fd(),
-                    seccomp_fd,
-                    status: None,
-                })
-            }
-            // SAFETY: This can never happen because clone child
-            // jumps to the specified function.
-            _ => unsafe { unreachable_unchecked() },
-        }
+        Ok(Child {
+            pid: child.into(),
+            pid_fd: pid_fd.as_raw_fd(),
+            seccomp_fd,
+            status: None,
+        })
     }
 
     fn after_start<Fd: AsFd>(mut self, pid: Pid, pid_fd: Fd) -> Result<RawFd, Errno> {
